@@ -166,7 +166,7 @@ class Auth:
             ServiceException: If the database update fails.
         """
         res = await self.db.update(tbl, kc, d)
-        if res!=1: log_and_raise_service(self.LOG, label='AUTH', label2=prefix, msg=f'DB update error: {prefix}')
+        if res!=1: log_and_raise_service(self.LOG, label='AUTH', label2=prefix, msg='DB update error: {prefix}')
 
     async def enforce_find_user(self, d, label2=''):
         """
@@ -531,6 +531,10 @@ class Auth:
         data = await request.json()
         [email, org] = enforce_required_args(self.LOG, data, ["email", "org"], label="AUTH", label2=label2, as_list=True)
         if type(org) == dict: org = oget(org, ["id"])
+
+        # exists_val = await self.db.execute("SELECT to_regclass('public.user_orgs')", fetchval=True)
+        # user_orgs_exists = exists_val is not None
+
         user2 = {
             "email": email,
             "role": "user",
@@ -558,7 +562,13 @@ class Auth:
             log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="USER_ALREADY_REGISTERED", status_code=401)
         user2["ustatus"] = NOT_REGISTERED
         # rec_to_row
-        await self.db.insert("users", user2)
+        uid= await self.db.insert("users", user2, return_key="id")
+
+        #if user_orgs_exists:
+        err = await self.db.insert("user_orgs", {"user_id": uid, "org_id": org})
+        if err:
+            self.LOG(3, 0, label="AUTH", label2=label2, msg="user_orgs insert skipped/duplicate")
+                
         return self.ret(auth_status="REGISTER_OK")
 
     # ####################################################################################################
@@ -625,14 +635,7 @@ class Auth:
             log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="USER_NOT_REGISTERED", status_code=400)
         if role == "admin" and user["org"] != org:
             log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="NOT_AUTHORIZED", status_code=403)
-        ustatus = SUSPENDED
-        val = rec.get("ustatus")
-        if val is not None:
-            try:
-                v = int(val)
-                ustatus = SUSPENDED_LICENCE if v==SUSPENDED_LICENCE else SUSPENDED
-            except (TypeError, ValueError):
-                pass
+        ustatus = int(rec["ustatus"]) if "ustatus" in rec and int(rec["ustatus"]) == SUSPENDED_LICENCE else SUSPENDED
         user["ustatus"] = ustatus
         await self.db_update("users", "id", user, label2)
         return rest_ok()
@@ -665,17 +668,69 @@ class Auth:
         jwt_token = mk_jwt_token(user["id"], user["email"], self.auth_secret)
         return rest_ok({"jwt_token": jwt_token, "user_info": get_user_info(user)})
 
+    # async def change_org(self, request):
+    #     """Change su org."""
+    #     label2 = "change_org"
+    #     _, _ = await self.enforce_admin_perm(request, label2=label2, requires_su=True)
+    #     if not is_numeric(request.user.display_name):
+    #         log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="USER_NOT_REGISTERED", status_code=400)
+    #     user = await self.db.select_one("users", {"id":int(request.user.display_name)})
+    #     if user is None:
+    #         log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="USER_NOT_REGISTERED", status_code=400)
+    #     data = await request.json()
+    #     user["org"] = oget(data,["org"])
+    #     jwt_token = mk_jwt_token(user["id"], user["email"], self.auth_secret)
+    #     _ = await self.db.update("users", "id", user)
+    #     return rest_ok({"jwt_token": jwt_token, "user_info": get_user_info(user)})
+    
     async def change_org(self, request):
-        """Change su org."""
+  
         label2 = "change_org"
-        _, _ = await self.enforce_admin_perm(request, label2=label2, requires_su=True)
+
+        ##  make sure the jwt user exists
         if not is_numeric(request.user.display_name):
-            log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="USER_NOT_REGISTERED", status_code=400)
-        user = await self.db.select_one("users", {"id":int(request.user.display_name)})
-        if user is None:
-            log_and_raise_service(self.LOG, label="AUTH", label2=label2, msg="USER_NOT_REGISTERED", status_code=400)
+            log_and_raise_service(self.LOG, label="AUTH", label2=label2,
+                                  msg="USER_NOT_REGISTERED", status_code=400)
+
+        uid = int(request.user.display_name)
+
+        # make sure that users exists and its active
+        user = await self.enforce_find_user({"id": uid}, label2=label2)
+        
+        # check if table user_orgs exists in schema public
+        # exists_val = await self.db.execute(
+        # "SELECT to_regclass('public.user_orgs')",
+        # fetchval=True
+        # )
+        # table_exists = exists_val is not None
+
+
+        # read target org from body
         data = await request.json()
-        user["org"] = oget(data,["org"])
+        target_org = oget(data, ["org"])
+        if target_org is None:
+            log_and_raise_service(self.LOG, label="AUTH", label2=label2,
+                                  msg="ORG_NOT_PROVIDED", status_code=400)
+
+        role = user.get("role")
+        ## if the table dont exist and the user is not su raise
+        # if not table_exists and role != "su":
+        #     log_and_raise_service(self.LOG, label="AUTH", label2=label2,
+        #                         msg="ORG_CHANGE_NOT_ALLOWED", status_code=403)
+        
+
+        # if not su, validate if the target_org is on user_orgs
+        if role != "su":
+            membership = await self.db.select_one("user_orgs", {"uid": uid, "org": target_org})
+            if membership is None:
+                log_and_raise_service(self.LOG, label="AUTH", label2=label2,
+                                      msg="NOT_AUTHORIZED_FOR_ORG", status_code=403)
+
+        # update org
+        user["org"] = target_org
+        await self.db_update("users", "id", user)
+
+        # new  token
         jwt_token = mk_jwt_token(user["id"], user["email"], self.auth_secret)
-        _ = await self.db.update("users", "id", user)
         return rest_ok({"jwt_token": jwt_token, "user_info": get_user_info(user)})
+
